function [ ] = driver_restoration( )
% -----------------------------------------------------------------------------
%
% NAME 
%
%   driver_restoration
%
% PURPOSE
%
%   Driver routine for a mass spectrum restoration example
%
% CALLING SEQUENCE
%
%   [ ] = driver_restoration( )
%
% INPUT PARAMETERS
%   
%   none
%
% OUTPUT PARAMETERS
%
%   none
%
% AUTHOR
%
%   (c) Johan De Keyser - Royal Belgian Institute for Space Aeronomy - 2019
%
% HISTORY
%
%   2019-05-15 : v0.0
%     Initial version
%   2019-06-18 : v1.0
%     Incorporated the effect of misalignment, slit width
%
% -----------------------------------------------------------------------------

    ButtonName = ...
        questdlg( ...
            'Please select', ...
            'Problem type', ...
            'Single mass peak', ...
            'Mass peak with shoulder', ...
            'Mass peak with shoulder' ...
        );
    switch ButtonName
        case 'Single mass peak'
            problem_type = 'single';
        case 'Mass peak with shoulder'
            problem_type = 'shoulder';
    end % switch
    
    ButtonName = ...
        questdlg( ...
            'Please select', ...
            'Mode selection', ...
            'Low mass resolution', ...
            'High mass resolution', ...
            'High mass resolution' ...
        );
    switch ButtonName
        case 'Low mass resolution'
            mode = 'LR';
            zoom = 6.4;
            slit_width = 6.5;
        case 'High mass resolution'
            mode = 'HR';
            zoom = 1;
            slit_width = 1;
    end % switch
    
    % Graphical parameters
    fs = 16;
    lw = 2.0;
    display_window = [ 290, 350 ];

    % Parameters
    np = 512;
    w_i1 = 2.0*slit_width/zoom;
    w_i2 = 3*w_i1;
    alpha_i = 0.00;
    w_e1 = 1.75;
    w_e2 = 4.0*w_e1;
    alpha_e = 0.05;
    dpixel = 1.0;
    zeta = 5;
    SN = 1000;
    
    % Ions present
    p = (1:np)';
    switch problem_type
        case 'single'
            fk = 100;
            pk = 323.3;
        case 'shoulder'
            fk = [ 100; 30 ];
            pk = [ 323.3; 316.7 ];
    end
    nk = length(pk);
    
    % Position-dependent gain correction
    depletion_position_left = 230;
    depletion_position_right = 320;
    depletion_half_width = 5;
    depletion_factor = 0.95;
    depletion_factor_centre = 0.25;
    Gpgc = ...
        create_Gpgc( ...
            p, ...
            depletion_factor, depletion_factor_centre, ...
            depletion_position_left, depletion_position_right, ...
            depletion_half_width ...
        );
        
    % Interpolate quantities on finer scale
    np_fine = zeta*np;
    p_fine = linspace(1-0.5,np+0.5,np_fine)';
    dp = (p_fine(2)-p_fine(1));
    Gpgc_fine = interp1( p, Gpgc, p_fine, 'pchip', 'extrap' );

    % Create LEDA spectrum at finest scale
    f_ion_fine = create_ion_spectrum( p_fine, pk, fk, w_i1, w_i2, alpha_i );
    f_LEDA_fine = create_LEDA_electron_spectrum( p_fine, f_ion_fine, w_e1, w_e2, alpha_e, dpixel, Gpgc_fine );
    f_LEDA_ideal_fine = create_ideal_electron_spectrum( p_fine, f_ion_fine, w_e1, w_e2, alpha_e, dpixel );
    
    % Create LEDA spectrum at nominal scale
    f_LEDA = grouped_sum( f_LEDA_fine, zeta );
    f_LEDA_ideal = grouped_sum( f_LEDA_ideal_fine, zeta );
        
    % Restoration
    f_LEDA_fine_to_be_restored = interp1( p, dp * f_LEDA, p_fine, 'pchip', 'extrap' );
    [ f_LEDA_fine_restored, f_ion_fine_restored ] = ...
        restore_LEDA_electron_spectrum( p_fine, f_LEDA_fine_to_be_restored, w_e1, w_e2, alpha_e, dpixel, Gpgc_fine, SN );
    f_LEDA_restored = grouped_sum( abs(f_LEDA_fine_restored), zeta );
    f_ion_restored = grouped_sum( abs(f_ion_fine_restored), zeta )/zeta;
    f_ion = grouped_sum( abs(f_ion_fine), zeta )/zeta;
    
    % Classic restoration
    f_LEDA_restored_classic = f_LEDA ./ Gpgc;
    
    selection = ( f_LEDA > 1e-3*max(f_LEDA) );
    ratio = abs(f_LEDA_restored_classic(selection))./f_LEDA_ideal(selection);
    the_std = std( ratio );
    the_max = max( ratio ) - 1;
    the_min = 1 - min( ratio );
    fprintf( 'LEDA Classic Restoration error: max %10.4f std %10.4f (relative error)\n', max([the_min,the_max]), the_std );
    
    selection = ( f_LEDA > 1e-3*max(f_LEDA) );
    ratio = abs(f_LEDA_restored(selection))./f_LEDA_ideal(selection);
    the_std = std( ratio );
    the_max = max( ratio ) - 1;
    the_min = 1 - min( ratio );
    fprintf( 'LEDA Restoration error: max %10.4f std %10.4f (relative error)\n', max([the_min,the_max]), the_std );
    
    selection = ( f_ion > 1e-3*max(f_ion) );
    ratio = abs(f_ion_restored(selection))./f_ion(selection);
    the_std = std( ratio );
    the_max = max( ratio ) - 1;
    the_min = 1 - min( ratio );
    fprintf( 'Ion Restoration error: max %10.4f std %10.4f (relative error)\n', max([the_min,the_max]), the_std );
        
    % Graphics
    h = findobj( 0, 'Type', 'figure', 'Tag', 'figure_restoration' );
    if isempty( h )
        h = figure( 'Tag', 'figure_restoration', 'Units', 'pixels', 'Position', [ 20 20 800 1200 ], 'PaperType', 'a4' );
    else
        figure(h);
        set( h, 'Units', 'pixels', 'Position', [ 20 20 800 1400 ] );
        clf;
    end
    ha1 = axes( h, 'box', 'on', 'Units', 'normalized', 'Position', [ 0.15 0.76 0.80, 0.20 ] );
    ha2 = axes( h, 'box', 'on', 'Units', 'normalized', 'Position', [ 0.15 0.54 0.80, 0.20 ] );
    ha3 = axes( h, 'box', 'on', 'Units', 'normalized', 'Position', [ 0.15 0.32 0.80, 0.20 ] );
    ha4 = axes( h, 'box', 'on', 'Units', 'normalized', 'Position', [ 0.15 0.10 0.80, 0.20 ] );
    
    % Position-dependent gain
    [ x1, y1 ] = boxplot( p, Gpgc );
    plot( ...
        ha1, ...
        x1, y1, 'b-', ...
        'Linewidth', lw ...
    );
    set( ha1, 'Box', 'on', 'Layer', 'top' );
    set( ha1, 'FontSize', fs, 'Linewidth', lw );
    set( ha1, 'XScale', 'linear', 'XLim', display_window, 'XTickLabel', {} );
    set( ha1, 'YScale', 'linear', 'YLim', [ 0 1 ] );
    ylabel( ha1, '$G_\mathrm{pgc}$', 'Interpreter', 'latex' );
    
    % Measured and ideal LEDA flux
    [ x1, y1 ] = boxplot( p, f_LEDA_ideal );
    [ x2, y2 ] = boxplot( p, f_LEDA );
    plot( ...
        ha2, ...
        x1, y1, 'g-', ...
        x2, y2, 'b-', ...
        'Linewidth', lw ...
    );
    hold( ha2, 'on' );
    for k = 1:nk
        plot( ...
            ha2, ...
            [ 1, 1 ] * pk(k), [1e-20,1e20], 'k:', ...
            'Linewidth', 0.5*lw ...
        );
    end
    set( ha2, 'Box', 'on', 'Layer', 'top' );
    set( ha2, 'FontSize', fs, 'Linewidth', lw );
    set( ha2, 'XScale', 'linear', 'XLim', display_window, 'XTickLabel', {} );
    set( ha2, 'YScale', 'log', 'YLim', [0.002, 5] * max(f_LEDA_ideal) );
    set_logarithmic_axis_labels( ha2 );
    ylabel( ha2, '$f$ [a.u.]', 'Interpreter', 'latex' ); 
    
    % Restoration ion flux and true ion flux
    [ x1, y1 ] = boxplot( p, f_ion_restored );
    plot( ...
        ha3, ...
        p_fine, f_ion_fine, 'g-', ...
        x1, y1, 'b-', ...
        'Linewidth', lw ...
    );
    hold( ha3, 'on' );
    for k = 1:nk
        plot( ...
            ha3, ...
            [ 1, 1 ] * pk(k), [1e-20,1e20], 'k:', ...
            'Linewidth', 0.5*lw ...
        );
    end
    set( ha3, 'Box', 'on', 'Layer', 'top' );
    set( ha3, 'FontSize', fs, 'Linewidth', lw );
    set( ha3, 'XScale', 'linear', 'XLim', display_window, 'XTickLabel', {} );
    set( ha3, 'YScale', 'log', 'YLim', [0.002, 5] * max(f_LEDA_ideal) );
    set_logarithmic_axis_labels( ha3 );
    ylabel( ha3, '$\hat{f}$ [a.u.]', 'Interpreter', 'latex' ); 
    
    % Restored and true electron spectrum
    [ x1, y1 ] = boxplot( p, f_LEDA_ideal );
    [ x2, y2 ] = boxplot( p, f_LEDA_restored );
    [ x3, y3 ] = boxplot( p, f_LEDA_restored_classic );
    plot( ...
        ha4, ...
        x1, y1, 'g-', ...
        x3, y3, 'm-', ...
        x2, y2, 'b-', ...
        'Linewidth', lw ...
    );
    set( ha4, 'Box', 'on', 'Layer', 'top' );
    set( ha4, 'FontSize', fs, 'Linewidth', lw );
    set( ha4, 'XScale', 'linear', 'XLim', display_window );
    set( ha4, 'YScale', 'log', 'YLim', [0.002, 5] * max(f_LEDA_ideal) );
    set_logarithmic_axis_labels( ha4 );
    ylabel( ha4, '$\tilde{f}$ [a.u.]', 'Interpreter', 'latex' );
    xlabel( ha4, '$p$ [pixel]', 'Interpreter', 'latex' )
    
    xpos = display_window(1) - 0.15 * ( display_window(2) - display_window(1) );
    lim = get( ha1, 'YLim' );
    text( ha1, xpos, lim(2), '(a)', 'HorizontalAlignment', 'left', 'VerticalAlignment','bottom', 'FontSize', fs );
    lim = get( ha2, 'YLim' );
    text( ha2, xpos, lim(2), '(b)', 'HorizontalAlignment', 'left', 'VerticalAlignment','bottom', 'FontSize', fs );
    lim = get( ha3, 'YLim' );
    text( ha3, xpos, lim(2), '(c)', 'HorizontalAlignment', 'left', 'VerticalAlignment','bottom', 'FontSize', fs );
    lim = get( ha4, 'YLim' );
    text( ha4, xpos, lim(2), '(d)', 'HorizontalAlignment', 'left', 'VerticalAlignment','bottom', 'FontSize', fs );
          
end

function [ tf, yf ] = boxplot( t, f )
    zeta = 20;
    n = length(t);
    dt = t(2)-t(1);
    nf = n*zeta;
    tf = linspace(t(1)-0.5*dt,t(n)+0.5*dt,nf)';
    yf = interp1( t, f, tf, 'nearest', 'extrap' );
end
    
function [] = set_logarithmic_axis_labels( ha )
    decades = [ -1:3];
    n_decades = length(decades);
    tick = zeros(9*n_decades,1);
    ticklabel = cell(9*n_decades,1);
    for i = 1:n_decades
        for j = 1:9
            tick(9*(i-1)+j) = (j)*10^decades(i);
            if j == 1
                ticklabel{9*(i-1)+j} = sprintf( '10^{%d}', decades(i) );
            else
                ticklabel{9*(i-1)+j} = '';
            end
        end
    end        
    set( ha, 'YTick', tick, 'YTickLabel', ticklabel );
end

% =========================================================================

function [ A ] = grouped_sum( a, zeta )
% -------------------------------------------------------------------------
%
% NAME
%
%	grouped_sum
%
% PURPOSE
%
%	This routine coarsens a vector by summing fixed numbers of adjacent elements.
%
% CALLING SEQUENCE
%
%	[ A ] = grouped_sum( a, zeta )
%
% INPUT PARAMETERS
%   
%   a
%     vector
%
%   zeta
%     integer, number of elements to group
%
% OUTPUT PARAMETERS
%
%   A
%     vector, smaller than a
%
% AUTHOR
%
%   (c) Johan De Keyser - Royal Belgian Institute for Space Aeronomy - 2019
%
% HISTORY
%
%   2019-05-15 : v0.0
%     Initial version
%
% -------------------------------------------------------------------------

    na = length(a);
    nA = ceil(na/zeta);
    A = zeros( [ nA, 1 ] );
    k = 1;
    m = 0;
    for i = 1:na
        A(k) = A(k) + a(i);
        m = m + 1;
        if m == zeta
            % you have summed zeta elements
            m = 0;
            k = k + 1;
        end
    end

end

% =========================================================================

